;; Generated package description from eldoc.el  -*- no-byte-compile: t -*-
(define-package "eldoc" "1.16.0.0.20251022.171210" "Show function arglist or variable docstring in echo area" '((emacs "26.3")) :commit "db6ac3c5f4ae3723f5772134d75b2625854e342d" :url "https://elpa.gnu.org/packages/eldoc.html" :authors '(("Noah Friedman" . "friedman@splode.com")) :maintainer '("Noah Friedman" . "friedman@splode.com") :keywords '("extensions"))
